package uy.edu.fing.tse.practico.db;

import uy.edu.fing.tse.dto.NoticiaDTO;
import uy.edu.fing.tse.dto.PublicacionDTO;
import uy.edu.fing.tse.practico.db.entity.Noticia;
import uy.edu.fing.tse.practico.db.entity.Publicacion;
import uy.edu.fing.tse.practico.db.mapper.Mapper;

import javax.annotation.PostConstruct;
import javax.ejb.Local;
import javax.ejb.LocalBean;
import javax.ejb.Remote;
import javax.ejb.Singleton;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import java.util.List;
import java.util.stream.Collectors;

@Singleton
@Local({DataAccessLocal.class})
@Remote({DataAccessRemote.class})
@LocalBean
public class DataAccessBean implements DataAccessLocal, DataAccessRemote {

    @PersistenceContext(unitName = "practico")
    private EntityManager em;

    @PostConstruct
    void init() {
        //Despues del constructor hace esto
    }

    @Override
    public NoticiaDTO alta(NoticiaDTO noticia) {
        final Noticia newN = em.merge(Mapper.mapper(noticia));
        em.flush();
        return Mapper.mapper(newN);
    }

    @Override
    public PublicacionDTO alta(PublicacionDTO publicacion) {
        final Publicacion newP = em.merge(Mapper.mapper(publicacion));
        em.flush();
        return Mapper.mapper(newP);
    }

    @Override
    public NoticiaDTO find(long idNoticia) {
        final TypedQuery<Noticia> query = em.createNamedQuery("Noticia.findById", Noticia.class);
        query.setParameter("id", idNoticia);
        final Noticia result = query.getSingleResult();
        return Mapper.mapper(result);
    }

    @Override
    public List<NoticiaDTO> getNoticias() {
        final TypedQuery<Noticia> query = em.createNamedQuery("Noticia.findAll", Noticia.class);
        return query.getResultList().stream().map(Mapper::mapper).collect(Collectors.toList());
    }

    @Override
    public List<PublicacionDTO> getPublicaciones() {
        final TypedQuery<Publicacion> query = em.createNamedQuery("Publicacion.findAll", Publicacion.class);
        return query.getResultList().stream().map(Mapper::mapper).collect(Collectors.toList());
    }

    @Override
    public NoticiaDTO addPublicacionToNoticia(final PublicacionDTO pub, final long idNoticia) {
        Noticia noticiaManaged = em.find(Noticia.class, idNoticia);

        if (noticiaManaged == null) {
            throw new RuntimeException("No existe Noticia");
        }

        Publicacion publicacion = Mapper.mapper(pub);
        publicacion = em.merge(publicacion);
        em.flush();

        noticiaManaged.addPublicacion(publicacion);
        noticiaManaged = em.merge(noticiaManaged);
        em.flush();

        return Mapper.mapper(noticiaManaged);
    }
}
